﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Globalization;
using C1.Win.C1Gauge;
using C1.Win.C1FlexGrid;
using C1.Win.Chart;

namespace BulletGraphs
{
    public partial class Form1 : Form
    {
        C1BulletGraph _bulletGraph;
        List<CategorySales> _categorySales;
        List<SalesData> _salesData;
        Random _random;
        public Form1()
        {
            InitializeComponent();
            this.Load += Form1_Load;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            CreateData();            

            ConfigureKeyMetricsBulletGraphs();

            ConfigureFlexGrid();

            ConfigureFlexChart();

            ConfigureRegionalSalesBulletGraphs();

            mainPanel.BackColor = Color.LightSteelBlue;
        }

        private void CreateData()
        {
            _salesData = new List<SalesData>();
            _random = new Random();
            string[] categories = { "Beverages", "Condiments", "Confections", "Dairy Products", "Sea food", "Cereals" };
            string[] regions = { "North", "South", "East", "West" };

            string[] months = DateTimeFormatInfo.CurrentInfo.AbbreviatedMonthNames;
            int sales;
            double expenses;
            for (int i = 0; i < (months.Length - 1) * categories.Length * regions.Length; i++)
            {
                sales = _random.Next(0, 100);
                expenses = sales - _random.Next(0, sales);
                _salesData.Add(new SalesData() { Category = categories[i / ((months.Length - 1) * regions.Length)], Region = regions[(i / (months.Length - 1)) % 4], Sales = sales, Expenses = expenses, LastYearSales = Math.Round(_random.NextDouble() * 100, 0), Month = months[i % (months.Length - 1)] });
            }
        }

        private void ConfigureKeyMetricsBulletGraphs()
        {
            double totalRevenue = _salesData.Sum(x => x.Sales);
            double totalExpenses = _salesData.Sum(x => x.Expenses);
            double totalProfit = totalRevenue - totalExpenses;
            List<KeyMetric> keyMetrics = new List<KeyMetric>();
            keyMetrics.Add(new KeyMetric() { MetricName = "Revenue", Value = totalRevenue, TargetValue = 12000, GoodValue= 10000, BadValue=5000});
            keyMetrics.Add(new KeyMetric() { MetricName = "Expenses", Value = totalExpenses, TargetValue = 4000, GoodValue=10000,BadValue=16000 });
            keyMetrics.Add(new KeyMetric() { MetricName = "Profit", Value = totalProfit, TargetValue = 8000,GoodValue=7000,BadValue=3000 });

            C1BulletGraph[] keyMetricBulletGraphs = { revenueBulletGraph, expensesBulletGraph, profitBulletGraph };
            C1BulletGraph bulletGraph;
            for (int i=0;i<keyMetrics.Count;i++)
            {
                bulletGraph = keyMetricBulletGraphs[i];
                //Sets the values of the featured measure and comparative measure for the BulletGraph
                bulletGraph.Value = keyMetrics[i].Value;
                bulletGraph.Target = keyMetrics[i].TargetValue;
                //Customizes the quantitative scale 
                bulletGraph.Minimum = 0;
                bulletGraph.Maximum = 16000;
                bulletGraph.GraphScale.LabelsInterval = 3000;
                bulletGraph.GraphScale.MarksInterval = 3000;
                bulletGraph.IsReversed = true;
                //Orients the BulletGraph vertically
                bulletGraph.Orientation = C1GaugeOrientation.Vertical;
                //Defines the qualitative ranges for the BulletGraph
                bulletGraph.Good.To = keyMetrics[i].GoodValue;
                bulletGraph.Bad.To = keyMetrics[i].BadValue;
                //Customizes the BulletGraph Caption
                bulletGraph.Caption.Text = keyMetrics[i].MetricName;
                bulletGraph.Caption.Alignment = StringAlignment.Near;
                //Styling the BulletGraph               
                bulletGraph.Styles.Labels.Caption.Font = new Font("Verdana", 10f, FontStyle.Bold);
                bulletGraph.Styles.Common.GraphOffset = -10;
                bulletGraph.Styles.Marks.Scale.Width =10;
            }
            //Customizes the starting value of the Good and Bad ranges for the BulletGraph depicting expenses( as expenses are good when low rather than high)
            keyMetricBulletGraphs[1].Good.From =5000;
            keyMetricBulletGraphs[1].Bad.From = 10000;
        }
        private void ConfigureFlexGrid()
        {
            _categorySales = _salesData.GroupBy(x => new
            {
                Category = x.Category,
            }).Select(y => new CategorySales()
            {
                Category = y.Key.Category,
                Sales = y.Sum(z => z.Sales),
                Target = _random.Next(2000, 5000),
                Good = _random.Next(2000, 4000),
                Bad = _random.Next(500, 2000)
            }).ToList<CategorySales>();
            c1FlexGrid1.DataSource = _categorySales;

            //Add an unbound column to the FlexGrid. This column will be used to show the BulletGraph control as an image 
            Column unboundCol = c1FlexGrid1.Cols.Add();
            unboundCol.Name = "SalesV/STarget";
            unboundCol.Caption = "SalesV/STarget";
            unboundCol.AllowEditing = false;

            //Set additional properties of the FlexGrid (optional)
            c1FlexGrid1.Cols.Fixed = 0;
            c1FlexGrid1.AllowEditing = false;
            c1FlexGrid1.Rows[0].StyleNew.Font = new Font(FontFamily.GenericSansSerif, 9f, FontStyle.Bold);
            c1FlexGrid1.Styles.Normal.Font = new Font(FontFamily.GenericSansSerif, 9f, FontStyle.Regular);
            c1FlexGrid1.Rows.DefaultSize = 38;
            c1FlexGrid1.Cols.DefaultSize = 82;
            c1FlexGrid1.Cols["Good"].Visible = false;
            c1FlexGrid1.Cols["Bad"].Visible = false;
            c1FlexGrid1.ExtendLastCol = true;
            c1FlexGrid1.Cols["Sales"].Caption = "Sales\n(in millions)";
            c1FlexGrid1.Cols["Target"].Caption = "Target\n(in millions)";
            c1FlexGrid1.Cols["Bad"].Caption = "Bad\n(in millions)";
            c1FlexGrid1.Cols["Good"].Caption = "Good\n(in millions)";
            for (int i = 0; i < c1FlexGrid1.Cols.Count; i++)
            {
                c1FlexGrid1.Cols[i].Format = "$ #,##0";
                c1FlexGrid1.Cols[i].TextAlign = TextAlignEnum.CenterCenter;
                c1FlexGrid1.Cols[i].TextAlignFixed = TextAlignEnum.CenterCenter;
            }

            //Subscribe to the OwnerDrawCell event
            c1FlexGrid1.DrawMode = C1.Win.C1FlexGrid.DrawModeEnum.OwnerDraw;
            c1FlexGrid1.OwnerDrawCell += C1FlexGrid1_OwnerDrawCell;

            //Sets the common properties of the BulletGraph control
            ConfigureFlexGridBulletGraph();
        }

        private void ConfigureFlexChart()
        {
            List<ChartData> chartData = _salesData.GroupBy(x => new
            {
                Month = x.Month,
            }).Select(y => new ChartData()
            {
                MonthName = y.Key.Month,
                CurrentYearSales = y.Sum(z => z.Sales),
                LastYearSales = y.Sum(z => z.LastYearSales),
            }).ToList<ChartData>();

            flexChart1.DataSource = chartData;
            flexChart1.BindingX = "MonthName";
            Series currentYrSeries = new Series();
            currentYrSeries.Name = "Current Year";
            currentYrSeries.Binding = "CurrentYearSales";
            currentYrSeries.ChartType = C1.Chart.ChartType.LineSymbols;
            flexChart1.Series.Add(currentYrSeries);

            Series previousYrSeries = new Series();
            previousYrSeries.Binding = "LastYearSales";
            previousYrSeries.Name = "Prior Year";
            previousYrSeries.ChartType = C1.Chart.ChartType.LineSymbols;
            flexChart1.Series.Add(previousYrSeries);

            flexChart1.AxisY.Title = "Sales (million $) ";
            flexChart1.AxisY.TitleStyle.Font = new Font("Arial", 12, FontStyle.Regular);
            flexChart1.AxisY.Min = 0;
            flexChart1.AxisY.MajorUnit = 300;
            flexChart1.AxisX.MajorUnit = 2;
            flexChart1.Legend.Position = C1.Chart.Position.Top;
        }

        private void ConfigureFlexGridBulletGraph()
        {
            //Configure the BulletGraph control
            _bulletGraph = new C1BulletGraph();
            _bulletGraph.Minimum = 0; //Sets the starting value of the quantitative scale
            _bulletGraph.Maximum = 5000; //Sets the ending value of the quantitative scale
            _bulletGraph.GraphScale.ShowLabels = false; //Hides the scale labels 
            _bulletGraph.GraphScale.ShowMarks = false; //Hides the scale tick marks

            //Styling the BulletGraph control
            _bulletGraph.BackColor = Color.White;
            _bulletGraph.Styles.Ranges.Value.Color = Color.SteelBlue;
            _bulletGraph.Styles.Ranges.Bar.Border.Color = Color.Gray;
            _bulletGraph.Styles.Ranges.Bar.Border.LineStyle = C1GaugeBorderStyle.Solid;
        }

        private void C1FlexGrid1_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
        {
            if (e.Row > 0 && c1FlexGrid1.Cols[e.Col].Name == "SalesV/STarget")
            {
                _bulletGraph.Bounds = e.Bounds;
                _bulletGraph.Value = _categorySales[e.Row - 1].Sales; //Sets the value of the featured measure
                _bulletGraph.Target = _categorySales[e.Row - 1].Target; //Sets the value of the comparative measure
                _bulletGraph.Bad.To = _categorySales[e.Row - 1].Bad; //Sets the ending value of the good range
                _bulletGraph.Good.To = _categorySales[e.Row - 1].Good; //Sets the ending value of the bad range

                //Retrieve the BulletGraph image and draw the retrieved image in the FlexGrid's cell
                Image bulletGraphImg = _bulletGraph.GetImage();
                e.Graphics.DrawImage(bulletGraphImg, e.Bounds);
                e.DrawCell(DrawCellFlags.Border);
                e.Handled = true;
            }
        }

        private void ConfigureRegionalSalesBulletGraphs()
        {
            List<RegionalSales> regionalSales = _salesData.GroupBy(x => new
            {
                Region = x.Region,
            }).Select(y => new RegionalSales()
            {
                Region = y.Key.Region,
                Sales = y.Sum(z => z.Sales),
                Target = _random.Next(2000, 5000),
                Good = _random.Next(2000, 4000),
                Bad = _random.Next(500, 2000)
            }).ToList<RegionalSales>();

            C1BulletGraph[] regionalSalesGraphs = { c1BulletGraph1, c1BulletGraph2, c1BulletGraph3, c1BulletGraph4 };
            for (int i = 0; i < regionalSales.Count; i++)
            {
                //Sets the values of the featured measure and comparative measure for the BulletGraph
                regionalSalesGraphs[i].Value = regionalSales[i].Sales;
                regionalSalesGraphs[i].Target = regionalSales[i].Target;
                //Customizes the quantitative scale 
                regionalSalesGraphs[i].Minimum = 0;
                regionalSalesGraphs[i].Maximum = 5000;
                regionalSalesGraphs[i].GraphScale.LabelsInterval = 1000;
                regionalSalesGraphs[i].GraphScale.MarksInterval = 1000;
                regionalSalesGraphs[i].GraphScale.Format = "$###0";
                //Defines the qualitative ranges for the Bullet Graph
                regionalSalesGraphs[i].Good.To = regionalSales[i].Good;
                regionalSalesGraphs[i].Bad.To = regionalSales[i].Bad;
                //Customizes the BulletGraph Caption
                regionalSalesGraphs[i].Caption.Text = regionalSales[i].Region;
                regionalSalesGraphs[i].Caption.Position = BulletGraphCaptionPosition.Left;
                regionalSalesGraphs[i].Caption.Alignment = StringAlignment.Far;
                //Styling the BulletGraph
                regionalSalesGraphs[i].Styles.Labels.Caption.FontSize = 30;
                if (regionalSalesGraphs[i].Value > regionalSalesGraphs[i].Target)
                    regionalSalesGraphs[i].Styles.Ranges.Value.Color = Color.ForestGreen;
                else
                    regionalSalesGraphs[i].Styles.Ranges.Value.Color = Color.Crimson;
            }
        }
    }
}
